@@ -1007,8 +1007,9 @@ void vcc_connected(VCC *vcc)
 		START_TIMER(entry,REPLY);
 		arp_request(entry->itf,entry->ip);
 	    }
-    if (vcc->entry->state != as_valid) inarp_request(vcc->entry);
-    else if (set_ip(vcc->fd,vcc->entry->ip) < 0)
+    inarp_request(vcc->entry);
+    if (vcc->entry->state == as_valid)
+	if (set_ip(vcc->fd,vcc->entry->ip) < 0)
 	    diag(COMPONENT,DIAG_ERROR,"can't set IP (%s)",strerror(errno));
 }
 
