@@ -9,10 +9,14 @@
 #include <stdint.h>
 #include <sys/socket.h> /* for struct sockaddr */
 #include <atm.h> /* for struct sockaddr_atmsvc */
+#include <atmd.h>
+
+#include "table.h"
 
 
 void open_all(void);
 void close_all(void);
+void notify(const UN_CTX *ctx,uint32_t ip,const ENTRY *entry);
 int do_close(int fd);
 void poll_loop(void);
 int connect_vcc(struct sockaddr *remote,const struct atm_qos *qos,int timeout);
