@@ -20,7 +20,7 @@ static void dump_required(FIELD *start,FIELD *leader,int group)
 
     for (walk = start; walk; walk = walk->next) {
 	if (walk->structure) {
-	    dump_required(walk->structure->block,leader,group);
+	    dump_required(walk->my_block,leader,group);
 	    continue;
 	}
 	if (walk->brk) break;
@@ -46,7 +46,7 @@ static void find_required(FIELD *start)
 
     for (walk = start; walk; walk = walk->next) {
 	if (walk->structure) {
-	    find_required(walk->structure->block);
+	    find_required(walk->my_block);
 	    continue;
 	}
 	if (walk->value) 
@@ -79,7 +79,7 @@ static void groups(FIELD *start,int group)
 
     for (walk = start; walk; walk = walk->next) {
 	if (walk->structure) {
-	    groups(walk->structure->block,group);
+	    groups(walk->my_block,group);
 	    continue;
 	}
 	if (walk->value)
@@ -89,11 +89,29 @@ static void groups(FIELD *start,int group)
 		case vt_case:
 		case vt_multi:
 		    for (scan = walk->value->tags; scan; scan = scan->next) {
+			int start_field,length,offset;
+
+			if (scan->block && scan->block->structure &&
+			  scan->block->structure->instances < 0) {
+			    start_field = scan->block->structure->first_field;
+			    length = scan->block->structure->fields;
+			    offset = scan->block->my_block->field-
+			      scan->block->structure->first_field;
+			}
+			else {
+			    start_field = -1;
+			    length = offset = 0;
+			}
 			if (scan->id) to_test("    \"%s\",\n",scan->id);
 			if (scan->block && scan->block->has_required)
-			    to_c("    { %d, required_%d }, /* %d: %s */ \n",
-			      group,scan->group,scan->group,scan->block->id);
-			else to_c("    { %d, NULL },\n",group);
+			    if (scan->block->id)
+				to_c("    { %d, required_%d, %d, %d, %d }, "
+				  "/* %s */\n",group,scan->group,start_field,
+				  length,offset,scan->block->id);
+			    else to_c("    { %d, required_%d, %d, %d, %d },\n",
+				  group,scan->group,start_field,length,offset);
+			else to_c("    { %d, NULL, %d, %d, %d },\n",group,
+			      start_field,length,offset);
 			groups(scan->block,scan->group);
 		    }
 		    break;
@@ -115,7 +133,7 @@ static void values(FIELD *start)
 
     for (walk = start; walk; walk = walk->next) {
 	if (walk->structure) {
-	    values(walk->structure->block);
+	    values(walk->my_block);
 	}
 	if (walk->value)
 	    switch (walk->value->type) {
@@ -159,7 +177,7 @@ static void fields(FIELD *start,int group)
 
     for (walk = start; walk; walk = walk->next) {
 	if (walk->structure) {
-	    fields(walk->structure->block,group);
+	    fields(walk->my_block,group);
 	    continue;
 	}
 	if (*walk->id != '_')
@@ -204,7 +222,7 @@ static void symbolic_names(FIELD *start)
 
     for (walk = start; walk; walk = walk->next) {
 	if (walk->structure) {
-	    symbolic_names(walk->structure->block);
+	    symbolic_names(walk->my_block);
 	    continue;
 	}
 	if (walk->name_list ? walk->name_list->id == -1 : !!walk->value) {
@@ -276,11 +294,12 @@ void second(FIELD *def)
     to_c("\n/*\n");
     to_c(" * If a group contains required fields, these are listed in the\n");
     to_c(" * following arrays. Each list ends with -1. The variable names\n");
-    to_c(" * contain the group number.\n */\n\n");
+    to_c(" * end with the group number.\n */\n\n");
     dump_required(def,def,0);
     find_required(def);
     to_c("\n/*\n * Various information about groups.\n */\n\n");
-    to_c("typedef struct {\n    int parent;\n    int *required;\n} GROUP;\n\n");
+    to_c("typedef struct {\n    int parent;\n    int *required;\n");
+    to_c("    int start;\n    int length;\n    int offset;\n} GROUP;\n\n");
     to_c("static GROUP groups[] = {\n");
     if (def->has_required) to_c("    { -1, required_0 },\n");
     else to_c("    { -1, NULL },\n");
@@ -289,7 +308,8 @@ void second(FIELD *def)
     to_test("    NULL\n};\n\n");
     to_c("};\n\n\n");
     to_c("/*\n * Named case selectors only have a limited set of valid\n");
-    to_c(" * values. They are listed in the following arrays.\n */\n\n");
+    to_c(" * values. They are listed in the following arrays, each followed\n");
+    to_c(" * by the number of the group it enables.\n */\n\n");
     values(def);
     to_c("\n/*\n * Various information about fields.\n */\n\n");
     to_c("typedef struct {\n    int parent;\n    int pos,size;\n");
