@@ -1,6 +1,6 @@
 /* kernel.c - Processing of incoming kernel messages */
  
-/* Written 1995-1998 by Werner Almesberger, EPFL-LRC/ICA */
+/* Written 1995-1999 by Werner Almesberger, EPFL-LRC/ICA */
  
 
 #include <stdlib.h>
@@ -57,8 +57,8 @@ static int send_setup(SOCKET *sock)
 	sock->sig = route_remote(&sock->remote);
 	if (!sock->sig) error = -EHOSTUNREACH;
 	else {
-	    local = get_local(sock->sig->signaling_pvc.sap_addr.itf);
-	    if (!local) local = get_local(ATM_ITF_ANY);
+	    local = get_local(sock->sig);
+	    if (!local) local = get_local(NULL);
 	    if (local) sock->local = *local;
 	    else {
 		error = -EADDRNOTAVAIL;
@@ -89,7 +89,7 @@ static int send_setup(SOCKET *sock)
     q_assign(&dsc,QF_sscs_type,0); /* unspecified - LANE wants this */
     if (!error)
 	error = sap_encode(&dsc,&sock->remote,&sock->sap,&sock->qos,
-	  sock->sig->uni);
+	  sock->sig->uni,sock->sig->max_rate);
     q_assign(&dsc,QF_bearer_class,16); /* BCOB-X */
     q_assign(&dsc,QF_upcc,0); /* p2p */
 #if !defined(UNI30) || defined(DYNAMIC_UNI)
@@ -198,12 +198,11 @@ static void dispatch(SOCKET *sock,struct atmsvc_msg *msg)
 	    }
 	    if (!atmsvc_addr_in_use(msg->svc))
 #ifdef BE_PICKY_ABOUT_BINDING_LOCAL_WILDCARD_ADDRESSES
-		if (local_addr[0].state != ls_same)
-		    SEND_ERROR(msg->vcc,-EADDRNOTAVAIL);
+		if (!get_local(NULL)) SEND_ERROR(msg->vcc,-EADDRNOTAVAIL);
 		else
 #endif
-		send_kernel(msg->vcc,0,as_okay,0,NULL,NULL,
-		      &local_addr[0].addr,NULL,NULL);
+		send_kernel(msg->vcc,0,as_okay,0,NULL,NULL,get_local(NULL),
+		  NULL,NULL);
 	    else {
 		if (!route_local(&msg->svc)) {
 		    SEND_ERROR(msg->vcc,-EADDRNOTAVAIL);
@@ -500,36 +499,36 @@ static void dispatch_listen(SOCKET *sock,struct atmsvc_msg *msg)
 }
 
 
-void itf_load(int itf)
+void sync_addr(VPCI *vpci)
 {
     char buf[MAX_ATM_ADDR_LEN+1];
     LOCAL_ADDR *walk;
 
-    (void) get_addr(itf);
-    for (walk = local_addr; walk->state != ls_unused; walk++)
-	if (walk->itf == itf) {
-	    if (atm2text(buf,MAX_ATM_ADDR_LEN+1,(struct sockaddr *)
-	      &walk->addr,pretty) < 0) strcpy(buf,"<invalid>");
-	    switch (walk->state) {
-		case ls_added:
-		    diag(COMPONENT,DIAG_INFO,"Added local ATM address %s at "
-		      "itf %d",buf,itf);
-		    walk->state = ls_same;
-		    break;
-		case ls_removed:
-		    diag(COMPONENT,DIAG_INFO,"Removed local ATM address %s at "
-		      "itf %d",buf,itf);
-		    walk->state = ls_unused;
+    (void) get_addr(vpci->itf,vpci->local_addr);
+    for (walk = vpci->local_addr; walk->state != ls_unused; walk++) {
+	if (atm2text(buf,MAX_ATM_ADDR_LEN+1,(struct sockaddr *)
+	  &walk->addr,pretty) < 0) strcpy(buf,"<invalid>");
+	switch (walk->state) {
+	    case ls_added:
+		diag(COMPONENT,DIAG_INFO,"Added local ATM address %s at "
+		  "itf %d",buf,vpci->itf);
+		walk->state = ls_same;
+		break;
+	    case ls_removed:
+		diag(COMPONENT,DIAG_INFO,"Removed local ATM address %s at "
+		  "itf %d",buf,vpci->itf);
 /*
+ *		walk->state = ls_unused;
+ *
  * This is probably wrong. What if we delete an entry from the middle of
  * the list ? @@@
  */
-		    /* @@@ delete SVCs using that address ? */
-		    break;
-		default:
-		    break;
-	    }
+		/* @@@ delete SVCs using that address ? */
+		break;
+	    default:
+		break;
 	}
+    }
 }
 
 
@@ -539,7 +538,7 @@ void from_kernel(struct atmsvc_msg *msg,int size)
     SOCKET *curr;
 
     if (msg->type == as_itf_notify) {
-	itf_load(msg->pvc.sap_addr.itf);
+	itf_reload(msg->pvc.sap_addr.itf);
 	return;
     }
     if (msg->type == as_terminate) {
